@@ -3,7 +3,7 @@
 addons="WEBIF WEBIF_LIVELOG WEBIF_JQUERY TOUCH WITH_SSL HAVE_DVBAPI READ_SDT_CHARSETS IRDETO_GUESSING CS_ANTICASC WITH_DEBUG MODULE_MONITOR WITH_LB CS_CACHEEX CW_CYCLE_CHECK LCDSUPPORT LEDSUPPORT CLOCKFIX IPV6SUPPORT"
 protocols="MODULE_CAMD33 MODULE_CAMD35 MODULE_CAMD35_TCP MODULE_NEWCAMD MODULE_CCCAM MODULE_CCCSHARE MODULE_GBOX MODULE_RADEGAST MODULE_SCAM MODULE_SERIAL MODULE_CONSTCW MODULE_PANDORA MODULE_GHTTP"
 readers="READER_NAGRA READER_IRDETO READER_CONAX READER_CRYPTOWORKS READER_SECA READER_VIACCESS READER_VIDEOGUARD READER_DRE READER_TONGFANG READER_BULCRYPT READER_GRIFFIN READER_DGCRYPT"
-card_readers="CARDREADER_PHOENIX CARDREADER_INTERNAL CARDREADER_SC8IN1 CARDREADER_MP35 CARDREADER_SMARGO CARDREADER_DB2COM CARDREADER_STAPI CARDREADER_STAPI5 CARDREADER_STINGER"
+card_readers="CARDREADER_PHOENIX CARDREADER_INTERNAL CARDREADER_SC8IN1 CARDREADER_MP35 CARDREADER_SMARGO CARDREADER_DB2COM CARDREADER_STAPI CARDREADER_STAPI5 CARDREADER_STINGER CARDREADER_DRECAS"
 
 defconfig="
 CONFIG_WEBIF=y
@@ -51,6 +51,7 @@ CONFIG_READER_BULCRYPT=y
 CONFIG_READER_GRIFFIN=y
 CONFIG_READER_DGCRYPT=y
 CARDREADER_PHOENIX=y
+CARDREADER_DRECAS=y
 CARDREADER_INTERNAL=y
 CARDREADER_SC8IN1=y
 CARDREADER_MP35=y
@@ -523,6 +524,7 @@ menu_card_readers() {
 		CARDREADER_STAPI	"STAPI"							$(check_test "CARDREADER_STAPI") \
 		CARDREADER_STAPI5	"STAPI5"						$(check_test "CARDREADER_STAPI5") \
 		CARDREADER_STINGER	"STINGER"						$(check_test "CARDREADER_STINGER") \
+		CARDREADER_DRECAS	"DRECAS"						$(check_test "CARDREADER_DRECAS") \
 	2> ${tempfile}
 
 	opt=${?}
@@ -562,7 +564,7 @@ config_dialog() {
 		opt=${?}
 		if [ $opt != 0 ]; then clear; exit; fi
 
-		menuitem=`cat $tempfile`
+		menuitem=$(cat $tempfile)
 		case $menuitem in
 			Add-ons) menu_addons ;;
 			Protocols) menu_protocols ;;
@@ -687,7 +689,12 @@ do
 		break
 	;;
 	'-r'|'--oscam-revision')
-		(svnversion -n . 2>/dev/null || printf 0) | sed 's/.*://; s/[^0-9]*$//; s/^$/0/'
+		revision=`(svnversion -n . 2>/dev/null || printf 0) | sed 's/.*://; s/[^0-9]*$//; s/^$/0/'`
+		if [ "$revision" = "0" ] 
+		then
+			which git > /dev/null 2>&1 && revision=`git log -1 --pretty=%B | grep git-svn-id | sed -n -e 's/^.*trunk@\([0-9]*\) .*$/\1/p'`
+		fi
+		echo $revision
 		break
 	;;
 	'-O'|'--detect-osx-sdk-version')
