@@ -356,6 +356,8 @@ typedef unsigned char uchar;
 #define SAFE_ATTR_SETSTACKSIZE_NOLOG(a,b)
 #endif
 
+#define CHECK_BIT(var,pos) (((var) & (1<<(pos)))? 1 : 0)
+
 /* ===========================
  *         constants
  * =========================== */
@@ -389,8 +391,13 @@ typedef unsigned char uchar;
 // Support for multiple CWs per channel and other encryption algos
 //#define WITH_EXTENDED_CW 1
 
+#if defined(READER_DRE) || defined(READER_DRECAS) || defined(READER_VIACCESS)
+#define MAX_ECM_SIZE 1024
+#define MAX_EMM_SIZE 1024
+#else
 #define MAX_ECM_SIZE 596
 #define MAX_EMM_SIZE 512
+#endif
 
 #define CS_EMMCACHESIZE  512 //nr of EMMs that each reader will cache
 #define MSGLOGSIZE 64   //size of string buffer for a ecm to return messages
@@ -423,6 +430,7 @@ typedef unsigned char uchar;
 /////////////////// readers that do not reed baudrate setting and timings are guarded by reader itself (large buffer built in): AFTER R_SMART
 #define R_SMART     0x7 // Smartreader+
 #define R_PCSC      0x8 // PCSC
+#define R_DRECAS    0x9 // Reader DRECAS
 /////////////////// proxy readers after R_CS378X
 #define R_CAMD35    0x20  // Reader cascading camd 3.5x
 #define R_CAMD33    0x21  // Reader cascading camd 3.3x
@@ -596,6 +604,11 @@ enum {E2_GLOBAL = 0, E2_GROUP, E2_CAID, E2_IDENT, E2_CLASS, E2_CHID, E2_QUEUE, E
 #define CW_ALGO_MODE_ECB 0
 #define CW_ALGO_MODE_CBC 1
 
+#define SIZE_SHORTDAY 8
+#define MAXALLOWEDTF 1001 // 10 allowed time frame slots for everyday + all [(3 + 1 + 10*(12) + 1)*8]
+extern const char *shortDay[SIZE_SHORTDAY];
+extern const char *weekdstr;
+
 /* ===========================
  *      Default Values
  * =========================== */
@@ -611,8 +624,6 @@ enum {E2_GLOBAL = 0, E2_GROUP, E2_CAID, E2_IDENT, E2_CLASS, E2_CHID, E2_QUEUE, E
 #define DEFAULT_CC_RECONNECT 12000
 #define DEFAULT_CC_RECV_TIMEOUT 2000
 
-#define CS_GBOX_MAX_PROXY_CARDS	32
-
 #define DEFAULT_AC_USERS   -1 // Use global cfg
 #define DEFAULT_AC_PENALTY -1 // Use global cfg
 
@@ -881,6 +892,7 @@ struct s_module
 	// int32_t checktype (0=return connected, 1=return loadbalance-avail) return int
 	void (*c_idle)(void);               // Schlocke: called when reader is idle
 	void (*s_idle)(struct s_client *);
+	void (*s_peer_idle)(struct s_client *);
 	void (*c_card_info)(void);              // Schlocke: request card infos
 
 	int32_t (*c_capmt)(struct s_client *, struct demux_s *);
@@ -1040,11 +1052,11 @@ typedef struct ecm_request_t
 	uint32_t        gbox_crc;       // rcrc for gbox, used to identify ECM task in peer responses
 	uint16_t        gbox_ecm_id;
 	uint8_t         gbox_ecm_status;
-	LLIST		*gbox_cards_pending; //type gbox_card_pending
+	LLIST			*gbox_cards_pending; //type gbox_card_pending
 #endif
 
 	void            *src_data;
-	int32_t         csp_hash; 					// csp has its own hash
+	uint32_t        csp_hash; 					// csp has its own hash
 
 	struct s_client *cacheex_src;               // Cacheex origin
 #ifdef CS_CACHEEX
@@ -1072,6 +1084,9 @@ typedef struct ecm_request_t
 #endif
 	struct ecm_request_t    *parent;
 	struct ecm_request_t    *next;
+#ifdef HAVE_DVBAPI
+	uint8_t		adapter_index;
+#endif
 } ECM_REQUEST;
 
 
@@ -1169,7 +1184,8 @@ struct s_client
 	time_t          lastemm;
 	time_t          lastecm;
 	time_t          expirationdate;
-	int32_t         allowedtimeframe[2];
+	uint32_t        allowedtimeframe[SIZE_SHORTDAY][24][2]; //day[0-sun to 6-sat, 7-ALL],hours,minutes use as binary flags to reduce mem usage
+	uint8_t         allowedtimeframe_set; //flag for internal use to mention if allowed time frame is used
 	int8_t          c35_suppresscmd08;
 	uint8_t         c35_sleepsend;
 	int8_t          ncd_keepalive;
@@ -1253,7 +1269,7 @@ struct s_client
 
 #ifdef MODULE_GBOX
 	void            *gbox;
-	uint16_t	gbox_peer_id;
+	uint16_t		gbox_peer_id;
 #endif
 
 #ifdef MODULE_GHTTP
@@ -1461,6 +1477,7 @@ struct s_reader                                     //contains device info, read
 	int8_t          fallback;
 	FTAB            fallback_percaid;
 	FTAB            localcards;
+	FTAB            disablecrccws_only_for;         // ignore checksum for selected caid provid
 #ifdef CS_CACHEEX
 	CECSP           cacheex; //CacheEx Settings
 #endif
@@ -1485,12 +1502,13 @@ struct s_reader                                     //contains device info, read
 	CAIDTAB         ctab;
 	uint32_t        boxid;
 	int8_t          nagra_read;                     // read nagra ncmed records: 0 Disabled (default), 1 read all records, 2 read valid records only
+	int8_t          detect_seca_nagra_tunneled_card;
 	int8_t          force_irdeto;
 	uint8_t         boxkey[16];                     // n3 boxkey 8 bytes, seca sessionkey 16 bytes, viaccess camid 4 bytes
 	uint8_t         boxkey_length;
 	uint8_t         rsa_mod[120];                   // rsa modulus for nagra cards.
 	uint8_t         rsa_mod_length;
-	uint8_t         des_key[32];                    // 3des key for Viaccess 16 bytes
+	uint8_t         des_key[128];                   // 3des key for Viaccess 16 bytes, des key for Dre 128 bytes
 	uint8_t         des_key_length;
 	uchar           atr[64];
 	uchar           card_atr[64];                   // ATR readed from card
@@ -1575,6 +1593,10 @@ struct s_reader                                     //contains device info, read
 	int32_t         emmskipped[4];                  // count skipped EMM
 	int32_t         emmerror[4];                    // count error EMM
 	int32_t         emmblocked[4];                  // count blocked EMM
+	int32_t         webif_emmwritten[4];            // count written EMM for webif reader info
+	int32_t         webif_emmskipped[4];            // count skipped EMM for webif reader info
+	int32_t         webif_emmerror[4];              // count error EMM for reader webif info
+	int32_t         webif_emmblocked[4];            // count blocked EMM for reader webif info
 	int32_t         lbvalue;                        // loadbalance Value
 #endif
 #ifdef WITH_AZBOX
@@ -1637,29 +1659,47 @@ struct s_reader                                     //contains device info, read
 	 	                   	       // (everything below 60 ms is converted to ms by applying *1000)
 	struct timeb    lastdvbapirateoverride;
 	uint32_t        ecmsok;
+	uint32_t        webif_ecmsok;
 	uint32_t        ecmsnok;
+	uint32_t        webif_ecmsnok;
+	uint32_t        ecmstout;
+	uint32_t        webif_ecmstout;
 	uint32_t        ecmnotfoundlimit;                   // config setting. restart reader if ecmsnok >= ecmnotfoundlimit
 	int32_t         ecmsfilteredhead;                   // count filtered ECM's by ECM Headerwhitelist
 	int32_t         ecmsfilteredlen;                    // count filtered ECM's by ECM Whitelist
+	int32_t         webif_ecmsfilteredhead;             // count filtered ECM's by ECM Headerwhitelist to readers ecminfo
+	int32_t         webif_ecmsfilteredlen;              // count filtered ECM's by ECM Whitelist to readers ecm info
 	float           ecmshealthok;
 	float           ecmshealthnok;
+	float           ecmshealthtout;
 	int32_t         cooldown[2];
 	int8_t          cooldownstate;
 	struct timeb    cooldowntime;
 	struct ecmrl    rlecmh[MAXECMRATELIMIT];
 	int8_t          fix_07;
 	int8_t          fix_9993;
-	int8_t			readtiers;							// method to get videoguard tiers
+	int8_t          readtiers; // method to get videoguard tiers
 	uint8_t         ins7E[0x1A + 1];
 	uint8_t         ins7E11[0x01 + 1];
 	uint8_t         ins2e06[0x04 + 1];
 	int8_t          ins7e11_fast_reset;
 	uint8_t         sc8in1_dtrrts_patch; // fix for kernel commit 6a1a82df91fa0eb1cc76069a9efe5714d087eccd
+
 #ifdef READER_VIACCESS	
-	unsigned char	initCA28; 							// To set when CA28 succeed
-	uint32_t 		key_schedule1[32];
-	uint32_t 		key_schedule2[32];
+	unsigned char   initCA28; // To set when CA28 succeed
+	uint32_t        key_schedule1[32];
+	uint32_t        key_schedule2[32];
 #endif
+
+#if defined(READER_DRE) || defined(READER_DRECAS)
+	char            *userscript;
+	uint32_t        force_ua;
+#endif
+
+#ifdef READER_DRECAS
+	char            *stmkeys;
+#endif
+
 #ifdef MODULE_GBOX
 	uint8_t		gbox_maxdist;
 	uint8_t		gbox_maxecmsend;
@@ -1701,6 +1741,8 @@ struct s_auth
 #ifdef CS_CACHEEX
 	CECSP           cacheex; //CacheEx Settings
 	uint8_t         no_wait_time;
+	uint8_t			disablecrccacheex;
+	FTAB 			disablecrccacheex_only_for;
 #endif
 	int16_t         allowedprotocols;
 	LLIST           *aureader_list;
@@ -1742,7 +1784,8 @@ struct s_auth
 	char            *dyndns;
 	time_t          expirationdate;
 	time_t          firstlogin;
-	int32_t         allowedtimeframe[2];
+	uint32_t        allowedtimeframe[SIZE_SHORTDAY][24][2]; //day[0-sun to 6-sat, 7-ALL],hours,minutes use as binary flags to reduce mem usage
+	uint8_t         allowedtimeframe_set; //flag for internal use to mention if allowed time frame is used
 	int8_t          c35_suppresscmd08;
 	uint8_t         c35_sleepsend;
 	int8_t          ncd_keepalive;
@@ -1910,6 +1953,8 @@ struct s_config
 	char            *emmlogdir;
 	char            *logfile;
 	char            *mailfile;
+	int8_t          disablecrccws;                  // 1=disable cw checksum test. 0=enable checksum check
+	FTAB            disablecrccws_only_for;         // ignore checksum for selected caid provid
 	uint8_t         logtostdout;
 	uint8_t         logtosyslog;
 	int8_t          logduplicatelines;
@@ -1961,13 +2006,14 @@ struct s_config
 	int8_t          http_showecminfo;
 	int8_t          http_showloadinfo;
 	int8_t          http_showuserinfo;
+	int8_t          http_showreaderinfo;
 	int8_t          http_showcacheexinfo;
 	struct s_ip     *http_allowed;
 	int8_t          http_readonly;
 	IN_ADDR_T       http_dynip[MAX_HTTP_DYNDNS];
 	uchar           http_dyndns[MAX_HTTP_DYNDNS][64];
 	int8_t          http_use_ssl;
-	int8_t          http_force_sslv3;
+	int8_t          https_force_secure_mode;
 	char            *http_cert;
 	char            *http_help_lang;
 	char            *http_locale;
@@ -2030,16 +2076,39 @@ struct s_config
 	uint32_t        cc_recv_timeout;                // The poll() timeout parameter in ms. Default: DEFAULT_CC_RECV_TIMEOUT (2000 ms).
 #endif
 #ifdef MODULE_GBOX
-	uint32_t        gbx_port[CS_MAXPORTS];
+    #define         GBOX_MY_VERS_DEF       0x2A
+    #define         GBOX_MY_CPU_API_DEF    0x40
+    #define         GBOX_MAX_PROXY_CARDS   32
+    #define         GBOX_MAX_IGNORED_PEERS 16
+    #define         GBOX_MAX_BLOCKED_ECM   16
+    #define         GBOX_MAX_REMM_PEERS    8
+    #define         GBOX_MAX_DEST_PEERS    16
+    #define         GBOX_MAX_MSG_TXT       127
+
+	uint16_t        gbox_port[CS_MAXPORTS];
 	char            *gbox_hostname;
-	int32_t         gbox_reconnect;
-	char            gbox_my_password[9];
-	unsigned long	gbox_proxy_card[CS_GBOX_MAX_PROXY_CARDS];
-	int8_t		gbox_proxy_cards_num;  
-	char            gbox_my_vers[3];
-	char		gbox_my_cpu_api[3];
-	uint8_t		gsms_dis;
-	char            *gbox_tmp_dir;      
+	uint32_t        gbox_reconnect;
+	uint32_t        gbox_password;
+	unsigned long   gbox_proxy_card[GBOX_MAX_PROXY_CARDS];
+	int8_t          gbox_proxy_cards_num;
+	uint32_t        gbox_my_vers;
+	uint8_t         gbox_my_cpu_api;
+	uint8_t         gsms_dis;
+	uint8_t         log_hello;
+	uint8_t         dis_attack_txt;
+	char            *gbox_tmp_dir; 
+	uint8_t         ccc_reshare;
+	uint16_t        gbox_ignored_peer[GBOX_MAX_IGNORED_PEERS];
+	uint8_t         gbox_ignored_peer_num;
+	uint16_t        accept_remm_peer[GBOX_MAX_REMM_PEERS];
+	uint8_t         accept_remm_peer_num;
+	uint16_t        gbox_block_ecm[GBOX_MAX_BLOCKED_ECM];
+	uint8_t         gbox_block_ecm_num;
+	uint8_t			gbox_save_gsms;
+	uint8_t			gbox_msg_type;
+	uint16_t		gbox_dest_peers[GBOX_MAX_DEST_PEERS];
+	uint8_t			gbox_dest_peers_num;
+	char			gbox_msg_txt[GBOX_MAX_MSG_TXT+1];
 #endif
 #ifdef MODULE_SERIAL
 	char            *ser_device;
@@ -2107,12 +2176,12 @@ struct s_config
 	int32_t     ac_denysamples;
 	char        *ac_logfile;
 	struct      s_cpmap *cpmap;
-	int8_t		acosc_enabled;
-	int8_t		acosc_max_active_sids;	// global value 0 - unlimited
-	int8_t		acosc_zap_limit;	// global value 0 - unlimited
-	int32_t		acosc_penalty_duration;	// global value how long is penalty activ in sek.
-	int8_t		acosc_penalty;	//global value
-	int32_t		acosc_delay;	//global value
+	int8_t      acosc_enabled;
+	int8_t      acosc_max_active_sids;// global value 0 - unlimited
+	int8_t      acosc_zap_limit;// global value 0 - unlimited
+	int32_t     acosc_penalty_duration;// global value how long is penalty activ in sek.
+	int8_t      acosc_penalty;//global value
+	int32_t     acosc_delay;//global value
 #endif
 
 #ifdef LEDSUPPORT
@@ -2144,6 +2213,15 @@ struct s_config
 	int32_t    max_cache_time;  //seconds ecms are stored in ecmcwcache
 	int32_t    max_hitcache_time;  //seconds hits are stored in cspec_hitcache (to detect dyn wait_time)
 
+	int8_t      reload_useraccounts;
+	int8_t      reload_readers;
+	int8_t      reload_provid;
+	int8_t      reload_services_ids;
+	int8_t      reload_tier_ids;
+	int8_t      reload_fakecws;
+	int8_t      reload_ac_stat;
+	int8_t      reload_log;
+
 	int8_t      block_same_ip;   //0=allow all, 1=block client requests to reader with same ip   (default=1)
 	int8_t      block_same_name; //0=allow all, 1=block client requests to reader with same name (default=1)
 
@@ -2168,7 +2246,7 @@ struct s_config
 	int8_t          cwcycle_dropold;        // what to do on old ecmd5/cw
 	int8_t          cwcycle_sensitive;
 	int8_t          cwcycle_allowbadfromffb;        //allow Bad cycles from Fixed Fallbackreader
-	int8_t			cwcycle_usecwcfromce;		//Use CWC Info from Cacheex Sources for CWC Checking
+	int8_t          cwcycle_usecwcfromce; //Use CWC Info from Cacheex Sources for CWC Checking
 #endif
 
 	//Global whitelist:
@@ -2310,7 +2388,7 @@ static inline bool caid_is_cryptoworks(uint16_t caid) { return caid >> 8 == 0x0D
 static inline bool caid_is_betacrypt(uint16_t caid) { return caid >> 8 == 0x17; }
 static inline bool caid_is_nagra(uint16_t caid) { return caid >> 8 == 0x18; }
 static inline bool caid_is_bulcrypt(uint16_t caid) { return caid == 0x5581 || caid == 0x4AEE; }
-static inline bool caid_is_dre(uint16_t caid) { return caid == 0x4AE0 || caid == 0x4AE1;}
+static inline bool caid_is_dre(uint16_t caid) { return caid == 0x4AE0 || caid == 0x4AE1 || caid == 0x2710;}
 const char *get_cardsystem_desc_by_caid(uint16_t caid);
 
 #endif
