@@ -655,24 +655,38 @@ static int32_t cacheex_add_to_cache_int(struct s_client *cl, ECM_REQUEST *er, in
 		cs_log_dbg(D_CACHEEX, "CACHEX received, but invalid client state %s", username(cl));
 		return 0;
 	}
-	
+
+	uint8_t selectedForIgnChecksum = chk_if_ignore_checksum(er, cfg.disablecrccws, &cfg.disablecrccws_only_for);
+			if (cl->typ == 'c') {
+				selectedForIgnChecksum += chk_if_ignore_checksum (er, cl->account->disablecrccacheex, &cl->account->disablecrccacheex_only_for);
+			}
+			if (cl->typ == 'p') {
+				selectedForIgnChecksum += chk_if_ignore_checksum (er, cl->reader->disablecrccws, &cl->reader->disablecrccws_only_for);
+			}
+
 	uint8_t i, c;
-	uint8_t null = 0;
-	for(i = 0; i < 16; i += 4)
+	if(cfg.disablecrccws == 0 || (cl->typ == 'c' && cl->account->disablecrccacheex == 0) || ( cl->typ == 'p' && cl->reader->disablecrccws == 0))
 	{
-		c = ((er->cw[i] + er->cw[i + 1] + er->cw[i + 2]) & 0xff);
-		null |= (er->cw[i] | er->cw[i + 1] | er->cw[i + 2]);
-		if(er->cw[i + 3] != c)
+		for(i = 0; i < 16; i += 4)
 		{
-			cs_log_dump_dbg(D_CACHEEX, er->cw, 16, "push received cw with chksum error from %s", csp ? "csp" : username(cl));
-			cl->cwcacheexerr++;
-			if(cl->account)
-				{ cl->account->cwcacheexerr++; }
-			return 0;
+			c = ((er->cw[i] + er->cw[i + 1] + er->cw[i + 2]) & 0xff);
+
+			if((i!=12) && selectedForIgnChecksum && (er->cw[i + 3] != c)){
+				break;
+			}
+
+			if(er->cw[i + 3] != c)
+			{
+				cs_log_dump_dbg(D_CACHEEX, er->cw, 16, "push received cw with chksum error from %s", csp ? "csp" : username(cl));
+				cl->cwcacheexerr++;
+				if(cl->account)
+					{ cl->account->cwcacheexerr++; }
+				return 0;
+			}
 		}
 	}
 
-	if(null == 0 || chk_is_null_CW(er->cw))
+	if(chk_is_null_CW(er->cw) && er->caid !=0x2600) // 0x2600 used by biss and constant cw could be indeed zero
 	{
 		cs_log_dump_dbg(D_CACHEEX, er->cw, 16, "push received null cw from %s", csp ? "csp" : username(cl));
 		cl->cwcacheexerr++;
