@@ -380,7 +380,7 @@ static void coolapi_read_data(dmx_t *dmx, dmx_callback_data_t *data)
 				continue;
 			}
 
-			dvbapi_process_input(dmx->demux_id, n, buffer, data->len);
+			dvbapi_process_input(dmx->demux_id, n, buffer, data->len, 0);
 		}
 	}
 }
@@ -688,6 +688,11 @@ int32_t coolapi_remove_filter(int32_t fd, int32_t num)
 		result = cnxt_dmx_close_filter(filter);
 		coolapi_check_error("cnxt_dmx_close_filter", result);
  	}
+	
+	// COOLAPI2 - We don't want to close Channel on no ECM Filters (Makes AU / EMMs work)
+	if(dmx->type != TYPE_ECM)
+		{ return 0; }
+
 	if (channel) {
 		result = cnxt_dmx_channel_close(channel);
 		coolapi_check_error("cnxt_dmx_channel_close", result);
