@@ -446,6 +446,20 @@ static READER_STAT *get_add_stat(struct s_reader *rdr, STAT_QUERY *q)
 
 static void housekeeping_stat(int32_t force);
 
+void readerinfofix_get_stat_query(ECM_REQUEST *er, STAT_QUERY *q)
+{
+	get_stat_query(er, q);
+}
+
+void readerinfofix_inc_fail(READER_STAT *s)
+{
+	inc_fail(s);
+}
+
+READER_STAT *readerinfofix_get_add_stat(struct s_reader *rdr, STAT_QUERY *q)
+{
+	return get_add_stat(rdr, q);
+}
 
 static int32_t get_reopen_seconds(READER_STAT *s)
 {
@@ -571,6 +585,9 @@ static void add_stat(struct s_reader *rdr, ECM_REQUEST *er, int32_t ecm_time, in
 	if((uint32_t)ecm_time >= 3 * cfg.ctimeout)
 		{ return; }
 
+	if((uint32_t)ecm_time >= cfg.ctimeout)
+		{ rc = E_TIMEOUT;}
+
 	STAT_QUERY q;
 	get_stat_query(er, &q);
 	READER_STAT *s;
@@ -1893,7 +1910,7 @@ static int8_t add_to_ecmlen(struct s_reader *rdr, READER_STAT *s)
 
 void update_ecmlen_from_stat(struct s_reader *rdr)
 {
-	if(!rdr || !&rdr->lb_stat)
+	if(!rdr || !rdr->lb_stat)
 		{ return; }
 
 	cs_readlock(__func__, &rdr->lb_stat_lock);
