@@ -27,7 +27,6 @@ uint32_t get_fallbacktimeout(uint16_t   caid)
 	return ftimeout;
 }
 
-
 static int32_t find_nano(uchar *ecm, int32_t l, uchar nano, int32_t s)
 {
 	uchar *snano;
@@ -51,34 +50,64 @@ static int32_t chk_class(ECM_REQUEST *er, CLASSTAB *clstab, const char *type, co
 	int32_t i, j, an, cl_n, l;
 	uchar ecm_class;
 
-	if(er->caid != 0x0500) { return 1; }
+	if(er->caid != 0x0500 && er->caid != 0x4AE1) { return 1; }
 	if(!clstab->bn && !clstab->an) { return 1; }
 
 	j = an = cl_n = l = 0;
-	while((j = find_nano(er->ecm, er->ecmlen, CS_NANO_CLASS, j)) > 0)
+	
+	if(er->caid == 0x0500)
 	{
-		l = er->ecm[j];
-		if(l + j > er->ecmlen) { continue; }  // skip, this is not a valid class identifier!
-		ecm_class = er->ecm[j + l];
-		cs_log_dbg(D_CLIENT, "ecm class=%02X", ecm_class);
-		for(i = 0; i < clstab->bn; i++)    // search in blocked
-			if(ecm_class == clstab->bclass[i])
-			{
-				cs_log_dbg(D_CLIENT, "class %02X rejected by %s '%s' !%02X filter",
-							  ecm_class, type, name, ecm_class);
-				return 0;
-			}
-
-		cl_n++;
-		for(i = 0; i < clstab->an; i++)    // search in allowed
-			if(ecm_class == clstab->aclass[i])
-			{
-				an++;
-				break;
-			}
-		j += l;
+		while((j = find_nano(er->ecm, er->ecmlen, CS_NANO_CLASS, j)) > 0)
+		{
+			l = er->ecm[j];
+			if(l + j > er->ecmlen) { continue; }  // skip, this is not a valid class identifier!
+			
+			ecm_class = er->ecm[j + l];
+			cs_log_dbg(D_CLIENT, "ecm class=%02X", ecm_class);
+			
+			for(i = 0; i < clstab->bn; i++)    // search in blocked
+				if(ecm_class == clstab->bclass[i])
+				{
+					cs_log_dbg(D_CLIENT, "class %02X rejected by %s '%s' !%02X filter",
+								  ecm_class, type, name, ecm_class);
+					return 0;
+				}
+  	
+			cl_n++;
+			for(i = 0; i < clstab->an; i++)    // search in allowed
+				if(ecm_class == clstab->aclass[i])
+				{
+					an++;
+					break;
+				}
+			j += l;
+		}
 	}
 
+	else
+	{
+		if(er->prid != 0x11 || er->ecm[0] == 0) { return 1; }
+		
+		cl_n++;
+		ecm_class = er->ecm[5];
+		cs_log_dbg(D_CLIENT, "ecm class=%02X", ecm_class);
+		
+		for(i = 0; i < clstab->bn; i++)    // search in blocked
+				if(ecm_class == clstab->bclass[i])
+				{
+					cs_log_dbg(D_CLIENT, "class %02X rejected by %s '%s' !%02X filter",
+								  ecm_class, type, name, ecm_class);
+					return 0;
+				}
+				
+		for(i = 0; i < clstab->an; i++)    // search in allowed
+				if(ecm_class == clstab->aclass[i])
+				{
+					an++;
+					break;
+				}
+	}
+	
 	if(cl_n && clstab->an)
 	{
 		if(an)
@@ -159,7 +188,6 @@ int32_t has_srvid(struct s_client *cl, ECM_REQUEST *er)
 	return 0;
 }
 
-
 int32_t has_lb_srvid(struct s_client *cl, ECM_REQUEST *er)
 {
 	if(!cl->lb_sidtabs.ok)
@@ -175,7 +203,6 @@ int32_t has_lb_srvid(struct s_client *cl, ECM_REQUEST *er)
 	return 0;
 }
 
-
 int32_t chk_srvid_match_by_caid_prov(uint16_t caid, uint32_t provid, SIDTAB *sidtab)
 {
 	int32_t i, rc = 0;
@@ -335,6 +362,7 @@ static int32_t chk_chid(ECM_REQUEST *er, FTAB *fchid, char *type, char *name)
 {
 	int32_t rc = 1, i, j, found_caid = 0;
 	if(!fchid->nfilts) { return 1; }
+	if(er->chid == 0 && er->ecm[0] == 0) { return 1; } // skip empty ecm, chid 00 to avoid no matching readers in dvbapi
 
 	for(i = rc = 0; (!rc) && i < fchid->nfilts; i++)
 		if(er->caid == fchid->filts[i].caid)
@@ -369,7 +397,6 @@ static int32_t chk_chid(ECM_REQUEST *er, FTAB *fchid, char *type, char *name)
 	return (rc);
 }
 
-
 int32_t chk_ident_filter(uint16_t rcaid, uint32_t rprid, FTAB *ftab)
 {
 	int32_t i, j, rc=1;
@@ -400,7 +427,6 @@ int32_t chk_ident_filter(uint16_t rcaid, uint32_t rprid, FTAB *ftab)
 	return(rc);
 }
 
-
 int32_t chk_ufilters(ECM_REQUEST *er)
 {
 	int32_t i, j, rc;
@@ -486,7 +512,7 @@ int32_t chk_rsfilter(struct s_reader *reader, ECM_REQUEST *er)
 							  (reader->prid[i][3]));
 			cs_log_dbg(D_CLIENT, "trying server '%s' filter %04X@%06X",
 						  reader->device, caid, prid);
-			if(prid == er->prid)
+			if(prid == er->prid || !er->prid)
 			{
 				rc = 1;
 				cs_log_dbg(D_CLIENT, "%04X@%06X allowed by server '%s' filter %04X@%06X",
@@ -543,7 +569,6 @@ int32_t chk_rfilter2(uint16_t rcaid, uint32_t rprid, struct s_reader *rdr)
 	return (rc);
 }
 
-
 static int32_t chk_rfilter(ECM_REQUEST *er, struct s_reader *rdr)
 {
 	return chk_rfilter2(er->caid, er->prid, rdr);
@@ -669,6 +694,33 @@ uint8_t chk_has_fixed_fallback(ECM_REQUEST *er)
 	return n_falb;
 }
 
+uint8_t chk_if_ignore_checksum(ECM_REQUEST *er, int8_t disablecrc, FTAB *disablecrc_only_for)
+{
+	if(!disablecrc && !disablecrc_only_for->nfilts) { return 0; }
+
+		int32_t i, k;
+		for(i = 0; i < disablecrc_only_for->nfilts; i++)
+		{
+			uint16_t tcaid = disablecrc_only_for->filts[i].caid;
+			if(tcaid && (tcaid == er->caid || (tcaid < 0x0100 && (er->caid >> 8) == tcaid)))    //caid match
+			{
+
+				int32_t nprids = disablecrc_only_for->filts[i].nprids;
+				if(!nprids)  // No Provider ->Ok
+					{ return 1; }
+
+				for(k = 0; k < nprids; k++)
+				{
+					uint32_t prid =disablecrc_only_for->filts[i].prids[k];
+					if(prid == er->prid)    //Provider matches
+					{ return 1; }
+				}
+			}
+		}
+
+		return 0;
+}
+
 int32_t matching_reader(ECM_REQUEST *er, struct s_reader *rdr)
 {
 
@@ -808,14 +860,16 @@ int32_t matching_reader(ECM_REQUEST *er, struct s_reader *rdr)
 		int8_t found = 0;
 		while((item = ll_iter_next(&itr)))
 		{
-			//if (item->caid == er->caid && (!er->prid || !item->provid || item->provid == er->prid)) {     //provid check causing problems?
-			if(item->caid == er->caid || item->caid == er->ocaid)                                           //... so check at least caid only
-			{
-				found = 1;
-				break;
-			}
+			if(item->caid != er->caid) continue; // skip wrong caid!
+			if(item->type == 7) continue; // skip seca-admin type (provid 000000) since its not used for decoding!
+			if(er->prid && item->provid && er->prid != item->provid) continue; // skip non matching provid!
+			if(!er->prid && caid_is_seca(er->caid)) continue; // dont accept requests without provid for seca cas.
+			if(!er->prid && caid_is_viaccess(er->caid)) continue; // dont accept requests without provid for viaccess cas
+			if(!er->prid && caid_is_cryptoworks(er->caid)) continue; // dont accept requests without provid for cryptoworks cas
+			found =1;
+			break;
 		}
-		if(!found)
+		if(!found && er->ecm[0]) // ecmrequest can get corrected provid parsed from payload in ecm
 		{
 			cs_log_dbg(D_TRACE, "entitlements check failed on reader %s", rdr->label);
 			return 0;
@@ -844,6 +898,7 @@ int32_t matching_reader(ECM_REQUEST *er, struct s_reader *rdr)
 		{
 			cs_log_dbg(D_TRACE, "ECM is not in ecmwhitelist of reader %s.", rdr->label);
 			rdr->ecmsfilteredlen += 1;
+			rdr->webif_ecmsfilteredlen += 1;
 			return (0);
 		}
 	}
@@ -923,10 +978,9 @@ int32_t matching_reader(ECM_REQUEST *er, struct s_reader *rdr)
 		{
 			if(skip == 0 || (foundcaid == 1 && foundprovid == 1 && entryok == 0 && skip == 1))
 			{
-				cs_log_dump_dbg(D_TRACE, er->ecm, er->ecmlen,
-							  "following ECM %04X@%06X:%04X was filtered by ECMHeaderwhitelist of Reader %s from User %s because of not matching Header:",
-							  er->caid, er->prid, er->srvid, rdr->label, username(er->client));
+				cs_log_dump_dbg(D_TRACE, er->ecm, er->ecmlen, "following ECM %04X@%06X:%04X was filtered by ECMHeaderwhitelist of Reader %s from User %s because of not matching Header:", er->caid, er->prid, er->srvid, rdr->label, username(er->client));
 				rdr->ecmsfilteredhead += 1;
+				rdr->webif_ecmsfilteredhead += 1;
 				return (0);
 			}
 		}
@@ -935,19 +989,15 @@ int32_t matching_reader(ECM_REQUEST *er, struct s_reader *rdr)
 	//Simple ring connection check:
 
 	//Check ip source+dest:
-	if(cfg.block_same_ip && IP_EQUAL(cur_cl->ip, rdr->client->ip) &&
-			get_module(cur_cl)->listenertype != LIS_DVBAPI &&
-			is_network_reader(rdr))
+	if(cfg.block_same_ip && IP_EQUAL(cur_cl->ip, rdr->client->ip) && get_module(cur_cl)->listenertype != LIS_DVBAPI && is_network_reader(rdr))
 	{
-		rdr_log_dbg(rdr, D_TRACE, "User (%s) has the same ip (%s) as the reader, blocked because block_same_ip=1!",
-					   username(cur_cl), cs_inet_ntoa(rdr->client->ip));
+		rdr_log_dbg(rdr, D_TRACE, "User (%s) has the same ip (%s) as the reader, blocked because block_same_ip=1!", username(cur_cl), cs_inet_ntoa(rdr->client->ip));
 		return 0;
 	}
 
 	if(cfg.block_same_name && strcmp(username(cur_cl), rdr->label) == 0)
 	{
-		rdr_log_dbg(rdr, D_TRACE, "User (%s) has the same name as the reader, blocked because block_same_name=1!",
-					   username(cur_cl));
+		rdr_log_dbg(rdr, D_TRACE, "User (%s) has the same name as the reader, blocked because block_same_name=1!", username(cur_cl));
 		return 0;
 	}
 
@@ -967,8 +1017,7 @@ int32_t chk_caid(uint16_t caid, CAIDTAB *ctab)
 	for(i = 0; i < ctab->ctnum; i++)
 	{
 		CAIDTAB_DATA *d = &ctab->ctdata[i];
-		if((caid & d->mask) == d->caid)
-			return d->cmap ? d->cmap : caid;
+		if((caid & d->mask) == d->caid) { return d->cmap ? d->cmap : caid; }
 	}
 	return -1;
 }
@@ -990,8 +1039,7 @@ int32_t chk_bcaid(ECM_REQUEST *er, CAIDTAB *ctab)
 {
 	int32_t caid;
 	caid = chk_caid(er->caid, ctab);
-	if(caid < 0)
-		{ return 0; }
+	if(caid < 0) { return 0; }
 	er->caid = caid;
 	return 1;
 }
@@ -1010,54 +1058,45 @@ int32_t chk_is_null_CW(uchar cw[])
 	return 1;
 }
 
-
-
 /**
  * Check for ecm request that expects half cw format
  **/
 int8_t is_halfCW_er(ECM_REQUEST *er)
 {
-	if(caid_is_videoguard(er->caid) &&
-	 (er->caid == 0x09C4 || er->caid ==  0x098C || er->caid == 0x0963 || er->caid == 0x09CD || er->caid == 0x0919 || er->caid == 0x093B || er->caid == 0x098E)
-	)
-		return 1;
-
-  return 0;
+	if( caid_is_videoguard(er->caid) && (er->caid == 0x09C4 || er->caid == 0x098C || er->caid == 0x098D || er->caid == 0x0963 || er->caid == 0x09CD || er->caid == 0x0919 || er->caid == 0x093B || er->caid == 0x098E))
+		{ return 1; }
+	return 0;
 }
 
-
 /**
  * Check for wrong half CWs
  **/
 int8_t chk_halfCW(ECM_REQUEST *er, uchar *cw)
 {
-  if(is_halfCW_er(er) && cw){
+	if(is_halfCW_er(er) && cw)
+	{
+		uchar cw15 = cw[15];
+		if(get_odd_even(er) == 0x80 && cw[15] == 0xF0) { cw[15] = 0; }
 
-	 int8_t part1 = checkCWpart(cw, 0);
-	 int8_t part2 = checkCWpart(cw, 1);
+		int8_t part1 = checkCWpart(cw, 0);
+		int8_t part2 = checkCWpart(cw, 1);
 
-	 //check for correct half cw format
-	 if(part1 && part2){
-		 return 0;
-	 }
+		//check for correct half cw format
+		if(part1 && part2){ cw[15] = cw15; return 0; }
 
-	 //check for correct cw position
-	 if(
-	    (get_odd_even(er) == 0x80 && part1 && !part2)   //xxxxxxxx00000000
+		//check for correct cw position
+		if(
+		(get_odd_even(er) == 0x80 && part1 && !part2)   //xxxxxxxx00000000
 		||
 		(get_odd_even(er) == 0x81 && !part1 && part2)   //00000000xxxxxxxx
-	 )
-	 {
-		return 1;
-	 }
-
-	 return 0;  //not correct swapped cw
-
-  }else
+		)
+		{ return 1; }
+		cw[15] = cw15;
+		return 0;  //not correct swapped cw
+	}else
 	return 1;
 }
 
-
 /**
  * Check for NULL nodeid
  **/
@@ -1066,8 +1105,7 @@ int32_t chk_is_null_nodeid(uint8_t node_id[], uint8_t len)
 	int8_t i;
 	for(i = 0; i < len; i++)
 	{
-		if(node_id[i])
-			{ return 0; }
+		if(node_id[i]) { return 0; }
 	}
 	return 1;
 }
@@ -1075,8 +1113,7 @@ int32_t chk_is_null_nodeid(uint8_t node_id[], uint8_t len)
 //check if client structure is accessible
 bool check_client(struct s_client *cl)
 {
-	if(cl && !cl->kill)
-		{ return true; }
+	if(cl && !cl->kill) { return true; }
 	return false;
 }
 
@@ -1086,8 +1123,7 @@ uint16_t caidvaluetab_get_value(CAIDVALUETAB *cv, uint16_t caid, uint16_t defaul
 	for(i = 0; i < cv->cvnum; i++)
 	{
 		CAIDVALUETAB_DATA *cvdata = &cv->cvdata[i];
-		if(cvdata->caid == caid || cvdata->caid == caid >> 8)
-			return cvdata->value;
+		if(cvdata->caid == caid || cvdata->caid == caid >> 8) { return cvdata->value; }
 	}
 	return default_value;
 }
@@ -1096,7 +1132,7 @@ int32_t chk_is_fakecw(uint8_t *cw)
 {
 	uint32_t i, is_fakecw = 0;
 	uint32_t idx = ((cw[0]&0xF)<<4) | (cw[8]&0xF); 
-	
+
 	cs_readlock(__func__, &config_lock);
 	for(i=0; i<cfg.fakecws[idx].count; i++)
 	{
