@@ -100,6 +100,7 @@ static void protocol_fn(const char *token, char *value, void *setting, FILE *f)
 			{ "gbox",       R_GBOX },
 			{ "cccam",      R_CCCAM },
 			{ "cccam_ext",  R_CCCAM },
+			{ "cccam_mcs",  R_CCCAM },
 			{ "constcw",    R_CONSTCW },
 			{ "radegast",   R_RADEGAST },
 			{ "scam",       R_SCAM },
@@ -107,6 +108,7 @@ static void protocol_fn(const char *token, char *value, void *setting, FILE *f)
 			{ "newcamd",    R_NEWCAMD },
 			{ "newcamd525", R_NEWCAMD },
 			{ "newcamd524", R_NEWCAMD },
+			{ "drecas",		R_DRECAS },
 			{ NULL        , 0 }
 		}, *p;
 		int i;
@@ -264,7 +266,7 @@ static void deskey_fn(const char *token, char *value, void *setting, FILE *f)
 	if(value)
 	{
 		int32_t len = strlen(value);
-		if(((len % 16) != 0) || len == 0 || len > 64)
+		if(((len % 16) != 0) || len == 0 || len > 128*2)
 		{
 			rdr->des_key_length = 0;
 			memset(rdr->des_key, 0, sizeof(rdr->des_key));
@@ -441,10 +443,11 @@ void ftab_fn(const char *token, char *value, void *setting, long ftab_type, FILE
 	if(ftab_type & FTAB_READER)
 	{
 		struct s_reader *rdr = NULL;
-		if(ftab_type & FTAB_PROVID)     { rdr = container_of(setting, struct s_reader, ftab); }
-		if(ftab_type & FTAB_CHID)       { rdr = container_of(setting, struct s_reader, fchid); }
-		if(ftab_type & FTAB_FBPCAID)    { rdr = container_of(setting, struct s_reader, fallback_percaid); }
-		if(ftab_type & FTAB_LOCALCARDS) { rdr = container_of(setting, struct s_reader, localcards); }
+		if(ftab_type & FTAB_PROVID)      { rdr = container_of(setting, struct s_reader, ftab); }
+		if(ftab_type & FTAB_CHID)        { rdr = container_of(setting, struct s_reader, fchid); }
+		if(ftab_type & FTAB_FBPCAID)     { rdr = container_of(setting, struct s_reader, fallback_percaid); }
+		if(ftab_type & FTAB_LOCALCARDS)  { rdr = container_of(setting, struct s_reader, localcards); }
+		if(ftab_type & FTAB_IGNCHKSMCAID){ rdr = container_of(setting, struct s_reader, disablecrccws_only_for); }
 		if(rdr)
 			{ rdr->changes_since_shareupdate = 1; }
 	}
@@ -629,6 +632,29 @@ static void ratelimitecm_fn(const char *token, char *value, void *setting, FILE
 		{ fprintf_conf(f, token, "%d\n", rdr->ratelimitecm); }
 }
 
+static void ecmunique_fn(const char *token, char *value, void *setting, FILE *f)
+{
+	struct s_reader *rdr = setting;
+	if(value)
+	{
+		if(strlen(value) == 0)
+		{
+			rdr->ecmunique = 0; // default
+		}
+		else
+		{
+			rdr->ecmunique = atoi(value);
+			if(rdr->ecmunique >= 1) 
+			{ rdr->ecmunique=1; }
+			else
+			{ rdr->ecmunique=0; }
+		}
+		return;
+	}
+	if((rdr->ratelimitecm && rdr->ecmunique!=0) || cfg.http_full_cfg)
+		{ fprintf_conf(f, token, "%d\n", rdr->ecmunique); }
+}
+
 static void ratelimittime_fn(const char *token, char *value, void *setting, FILE *f)
 {
 	struct s_reader *rdr = setting;
@@ -658,6 +684,7 @@ static void ratelimittime_fn(const char *token, char *value, void *setting, FILE
 	if(rdr->ratelimitecm || cfg.http_full_cfg)
 		{ fprintf_conf(f, token, "%d\n", rdr->ratelimittime); }
 }
+
 static void srvidholdtime_fn(const char *token, char *value, void *setting, FILE *f)
 {
 	struct s_reader *rdr = setting;
@@ -814,6 +841,7 @@ static const struct config_list reader_opts[] =
 	DEF_OPT_INT8("fallback"             , OFS(fallback),                0),
 	DEF_OPT_FUNC_X("fallback_percaid"   , OFS(fallback_percaid),        ftab_fn, FTAB_READER | FTAB_FBPCAID),
 	DEF_OPT_FUNC_X("localcards"         , OFS(localcards),        		ftab_fn, FTAB_READER | FTAB_LOCALCARDS),
+	DEF_OPT_FUNC_X("disablecrccws_only_for", OFS(disablecrccws_only_for),     ftab_fn, FTAB_READER | FTAB_IGNCHKSMCAID),
 #ifdef CS_CACHEEX
 	DEF_OPT_INT8("cacheex"              , OFS(cacheex.mode),            0),
 	DEF_OPT_INT8("cacheex_maxhop"       , OFS(cacheex.maxhop),          0),
@@ -842,6 +870,7 @@ static const struct config_list reader_opts[] =
 	DEF_OPT_FUNC("ecmheaderwhitelist"   , 0,                            ecmheaderwhitelist_fn),
 	DEF_OPT_FUNC("detect"               , 0,                            detect_fn),
 	DEF_OPT_INT8("nagra_read"           , OFS(nagra_read),              0),
+	DEF_OPT_INT8("detect_seca_nagra_tunneled_card", OFS(detect_seca_nagra_tunneled_card), 1),
 	DEF_OPT_INT32("mhz"                 , OFS(mhz),                     357),
 	DEF_OPT_INT32("cardmhz"             , OFS(cardmhz),                 357),
 #ifdef WITH_AZBOX
@@ -886,14 +915,21 @@ static const struct config_list reader_opts[] =
 #endif
 #ifdef MODULE_GHTTP
 	DEF_OPT_UINT8("use_ssl"             , OFS(ghttp_use_ssl),           0),
+#endif
+#if defined(READER_DRE) || defined(READER_DRECAS)
+	DEF_OPT_HEX("force_ua"              , OFS(force_ua),                4),
+	DEF_OPT_STR("exec_cmd_file"         , OFS(userscript),              NULL),
+#endif
+#ifdef READER_DRECAS
+	DEF_OPT_STR("stmkeys"               , OFS(stmkeys),                 NULL),
 #endif
 	DEF_OPT_INT8("deprecated"           , OFS(deprecated),              0),
 	DEF_OPT_INT8("audisabled"           , OFS(audisabled),              0),
 	DEF_OPT_FUNC("auprovid"             , 0,                            auprovid_fn),
 	DEF_OPT_INT8("ndsversion"           , OFS(ndsversion),              0),
 	DEF_OPT_FUNC("ratelimitecm"         , 0,                            ratelimitecm_fn),
+	DEF_OPT_FUNC("ecmunique"            , 0,                            ecmunique_fn),
 	DEF_OPT_FUNC("ratelimittime"        , 0,                            ratelimittime_fn),
-	DEF_OPT_INT8("ecmunique"            , OFS(ecmunique),               0),
 	DEF_OPT_FUNC("srvidholdtime"        , 0,                            srvidholdtime_fn),
 	DEF_OPT_FUNC("cooldown"             , 0,                            cooldown_fn),
 	DEF_OPT_FUNC("cooldowndelay"        , 0,                            cooldowndelay_fn),
@@ -922,6 +958,9 @@ static bool reader_check_setting(const struct config_list *UNUSED(clist), void *
 		"readnano", "resetcycle", "smargopatch", "autospeed", "sc8in1_dtrrts_patch", "boxid","fix07",
 		"fix9993", "rsakey", "deskey", "ins7e", "ins7e11", "ins2e06", "force_irdeto", "needsemmfirst", "boxkey",
 		"atr", "detect", "nagra_read", "mhz", "cardmhz", "readtiers", "read_old_classes",
+#if defined(READER_DRE) || defined(READER_DRECAS)
+		"exec_cmd_file",
+#endif
 #ifdef WITH_AZBOX
 		"mode",
 #endif
