@@ -538,13 +538,13 @@ ECM_REQUEST *get_ecmtask(void)
 
 void cleanup_ecmtasks(struct s_client *cl)
 {
-	if(cl && !cl->account->usr) { return; }  //not for anonymous users!
+	if(!cl) { return; }
 
 	ECM_REQUEST *ecm;
 
 	//remove this clients ecm from queue. because of cache, just null the client:
 	cs_readlock(__func__, &ecmcache_lock);
-	for(ecm = ecmcwcache; ecm; ecm = ecm->next)
+	for(ecm = ecmcwcache; ecm && cl; ecm = ecm->next)
 	{
 		if(ecm->client == cl)
 		{
@@ -561,7 +561,7 @@ void cleanup_ecmtasks(struct s_client *cl)
 		if(check_client(rdr->client) && rdr->client->ecmtask)
 		{
 			int i;
-			for(i = 0; i < cfg.max_pending; i++)
+			for(i = 0; (i < cfg.max_pending) && cl; i++)
 			{
 				ecm = &rdr->client->ecmtask[i];
 				if(ecm->client == cl)
@@ -674,7 +674,7 @@ int32_t send_dcw(struct s_client *client, ECM_REQUEST *er)
 								};
 	static const char *stxtEx[16] = {"", "group", "caid", "ident", "class", "chid", "queue", "peer", "sid", "", "", "", "", "", "", ""};
 	static const char *stxtWh[16] = {"", "user ", "reader ", "server ", "lserver ", "", "", "", "", "", "", "", "" , "" , "", ""};
-	char sby[100] = "", sreason[32] = "", scwcinfo[32] = "", schaninfo[32] = "", srealecmtime[50]="";
+	char sby[100] = "", sreason[32] = "", scwcinfo[32] = "", schaninfo[CS_SERVICENAME_SIZE] = "", srealecmtime[50]="";
 	char erEx[32] = "";
 	char usrname[38] = "";
 	char channame[CS_SERVICENAME_SIZE];
@@ -1028,7 +1028,7 @@ int32_t send_dcw(struct s_client *client, ECM_REQUEST *er)
 	}
 #endif
 
-	if(cfg.double_check && er->rc == E_FOUND && er->selected_reader && is_double_check_caid(er))
+	if(cfg.double_check && er->rc < E_NOTFOUND && er->selected_reader && is_double_check_caid(er))
 	{
 		if(er->checked == 0)   //First CW, save it and wait for next one
 		{
@@ -1042,11 +1042,12 @@ int32_t send_dcw(struct s_client *client, ECM_REQUEST *er)
 			if(memcmp(er->cw_checked, er->cw, sizeof(er->cw)) == 0)
 			{
 				er->checked++;
-				cs_log("DOUBLE CHECKED! %d. CW by %s idx %d cpti %d", er->checked, er->selected_reader->label, er->idx, er->msgid);
+				cs_log("CW matched by %s total matches %d idx %d cpti %d", er->selected_reader->label, er->checked, er->idx, er->msgid);
 			}
 			else
 			{
-				cs_log("DOUBLE CHECKED NONMATCHING! %d. CW by %s idx %d cpti %d", er->checked, er->selected_reader->label, er->idx, er->msgid);
+				er->checked--;
+				cs_log("CW mismatch by %s total matches %d idx %d cpti %d", er->selected_reader->label, er->checked, er->idx, er->msgid);
 			}
 		}
 		if(er->checked < 2)    //less as two same cw? mark as pending!
@@ -1215,7 +1216,7 @@ void request_cw_from_readers(ECM_REQUEST *er, uint8_t stop_stage)
 }
 
 
-void add_cache_from_reader(ECM_REQUEST *er, struct s_reader *rdr, int32_t csp_hash, uchar *ecmd5, uchar *cw, int16_t caid, int32_t prid, int16_t srvid ){
+void add_cache_from_reader(ECM_REQUEST *er, struct s_reader *rdr, uint32_t csp_hash, uchar *ecmd5, uchar *cw, int16_t caid, int32_t prid, int16_t srvid ){
 	ECM_REQUEST *ecm;
 	if (cs_malloc(&ecm, sizeof(ECM_REQUEST))){
 		cs_ftime(&ecm->tps);
@@ -1280,7 +1281,7 @@ void chk_dcw(struct s_ecm_answer *ea)
 			ECM_REQUEST *er = ert;
 			debug_ecm(D_TRACE, "WARNING2: Different CWs %s from %s(%s)<>%s(%s): %s<>%s", buf,
 					  username(ea->reader ? ea->reader->client : ert->client), ip1,
-					  er->cacheex_src ? username(er->cacheex_src) : (ea->reader ? ea->reader->label : "unknown/csp"), ip2,
+					  er->cacheex_src ? username(er->cacheex_src) : (ert->selected_reader ? ert->selected_reader->label : "unknown/csp"), ip2,
 					  cw1, cw2);
 		}
 #endif
@@ -1332,6 +1333,9 @@ void chk_dcw(struct s_ecm_answer *ea)
 		ert->rcEx = 0;
 		ert->rc = ea->rc;
 		ert->grp |= eardr->grp;
+#ifdef HAVE_DVBAPI
+		ert->adapter_index = ea->er->adapter_index;
+#endif
 
 		break;
 	case E_INVALID:
@@ -1565,24 +1569,36 @@ int32_t write_ecm_answer(struct s_reader *reader, ECM_REQUEST *er, int8_t rc, ui
 	}
 
 	//SPECIAL CHECKs for rc
-	if(rc < E_NOTFOUND && cw && chk_is_null_CW(cw))    //if cw=0 by anticascading
+	if(rc < E_NOTFOUND && cw && chk_is_null_CW(cw) && er->caid !=0x2600) // 0x2600 used by biss and constant cw could be zero but still catch cw=0 by anticascading
 	{
 		rc = E_NOTFOUND;
 		cs_log_dbg(D_TRACE | D_LB, "WARNING: reader %s send fake cw, set rc=E_NOTFOUND!", reader ? reader->label : "-");
 	}
 
-	if(rc < E_NOTFOUND && cw && !chk_halfCW(er,cw)){
-		rc = E_NOTFOUND;
-		cs_log_dbg(D_TRACE | D_LB, "WARNING: reader %s send wrong swapped NDS cw, set rc=E_NOTFOUND!", reader ? reader->label : "-");
-	}
+
+
+ 	if(rc < E_NOTFOUND && cw && !chk_halfCW(er,cw)){
+ 		rc = E_NOTFOUND;
+ 		cs_log_dbg(D_TRACE | D_LB, "WARNING: reader %s send wrong swapped NDS cw, set rc=E_NOTFOUND!", reader ? reader->label : "-");
+ 	}
+
 
 	if(reader && cw && rc < E_NOTFOUND)
 	{
-		if(reader->disablecrccws == 0)
+		if(cfg.disablecrccws == 0 && reader->disablecrccws == 0)
 		{
+			uint8_t selectedForIgnChecksum = chk_if_ignore_checksum(er, cfg.disablecrccws, &cfg.disablecrccws_only_for)
+					+ chk_if_ignore_checksum(er, reader->disablecrccws, &reader->disablecrccws_only_for);
+
 			for(i = 0; i < 16; i += 4)
 			{
 				c = ((cw[i] + cw[i + 1] + cw[i + 2]) & 0xff);
+
+				if((i!=12) && selectedForIgnChecksum && (cw[i + 3] != c)){
+					cs_log_dbg(D_TRACE, "notice: CW checksum check disabled for %04X:%06X", er->caid, er->prid);
+					break;
+				}
+
 				if(cw[i + 3] != c)
 				{
 					unsigned char nano = 0x00;
@@ -1679,7 +1695,7 @@ int32_t write_ecm_answer(struct s_reader *reader, ECM_REQUEST *er, int8_t rc, ui
 	if(!ea->is_pending)   //not for pending ea - only once for ea
 	{
 		//cache update
-		if(ea && ea->rc < E_NOTFOUND && ea->cw)
+		if(ea && (ea->rc < E_NOTFOUND) && (!chk_is_null_CW(ea->cw) && er->caid !=0x2600)) // 0x2600 used by biss and constant cw could be indeed zero
 			add_cache_from_reader(er, reader, er->csp_hash, er->ecmd5, ea->cw, er->caid, er->prid, er->srvid );
 
 		//readers stats for LB
@@ -1697,6 +1713,7 @@ int32_t write_ecm_answer(struct s_reader *reader, ECM_REQUEST *er, int8_t rc, ui
 				{ logCWtoFile(er, ea->cw); } /* CWL logging only if cwlogdir is set in config */
 
 			reader->ecmsok++;
+			reader->webif_ecmsok++;
 #ifdef CS_CACHEEX
 			struct s_client *eacl = reader->client;
 			if(cacheex_reader(reader) && check_client(eacl))
@@ -1710,6 +1727,7 @@ int32_t write_ecm_answer(struct s_reader *reader, ECM_REQUEST *er, int8_t rc, ui
 		else if(ea->rc == E_NOTFOUND)
 		{
 			reader->ecmsnok++;
+			reader->webif_ecmsnok++;
 			if(reader->ecmnotfoundlimit && reader->ecmsnok >= reader->ecmnotfoundlimit)
 			{
 				rdr_log(reader, "ECM not found limit reached %u. Restarting the reader.",
@@ -1720,11 +1738,30 @@ int32_t write_ecm_answer(struct s_reader *reader, ECM_REQUEST *er, int8_t rc, ui
 			}
 		}
 
-		//Reader ECMs Health Try (by Pickser)
-		if(reader->ecmsok != 0 || reader->ecmsnok != 0)
+		// this fixes big oscam mistake - wrong reader status on web info aka not counted timeouts which dispalyed reader info 100 percent OK but reader had a ton of unhandled timeouts!
+		else if(ea->rc == E_TIMEOUT)
 		{
-			reader->ecmshealthok = ((double) reader->ecmsok / (reader->ecmsok + reader->ecmsnok)) * 100;
-			reader->ecmshealthnok = ((double) reader->ecmsnok / (reader->ecmsok + reader->ecmsnok)) * 100;
+#ifdef WITH_LB
+			STAT_QUERY q;
+			readerinfofix_get_stat_query(er, &q);
+			READER_STAT *s;
+			s = readerinfofix_get_add_stat(reader, &q);
+			if (s)
+			{
+				cs_log_dbg(D_LB, "inc fail {client %s, caid %04X, prid %06X, srvid %04X} [write_ecm_answer] reader %s rc %d, ecm time %d ms (%d ms)", (check_client(er->client) ? er->client->account->usr : "-"), er->caid, er->prid, er->srvid, reader ? reader->label : "-", rc, ea->ecm_time, ntime);
+				readerinfofix_inc_fail(s); //now increase fail factor for unhandled timeouts
+			}
+#endif
+			reader->ecmstout++; //now append timeouts to the readerinfo timeout count
+			reader->webif_ecmstout++;
+		}
+
+		//Reader ECMs Health Try (by Pickser)
+		if(reader->ecmsok != 0 || reader->ecmsnok != 0 || reader->ecmstout != 0)
+		{
+			reader->ecmshealthok = ((double) reader->ecmsok / (reader->ecmsok + reader->ecmsnok + reader->ecmstout)) * 100;
+			reader->ecmshealthnok = ((double) reader->ecmsnok / (reader->ecmsok + reader->ecmsnok + reader->ecmstout)) * 100;
+			reader->ecmshealthtout = ((double) reader->ecmstout / (reader->ecmsok + reader->ecmsnok + reader->ecmstout)) * 100;
 		}
 
 		if(rc == E_FOUND && reader->resetcycle > 0)
@@ -1802,7 +1839,7 @@ uint32_t get_subid(ECM_REQUEST *er)
 		id = b2i(2, er->ecm + 11);
 		break; // videoguard
 	case 0x4A: // DRE-Crypt, Bulcrypt, Tongfang and others?
-		if(!caid_is_bulcrypt(er->caid))
+		if(!caid_is_bulcrypt(er->caid) && !caid_is_dre(er->caid))
 			{ id = b2i(2, er->ecm + 6); }
 		break;
 	}
@@ -2058,18 +2095,26 @@ void get_cw(struct s_client *client, ECM_REQUEST *er)
 		{ er->rc = E_EXPDATE; }
 
 	// out of timeframe
-	if(client->allowedtimeframe[0] && client->allowedtimeframe[1])
+	if(client->allowedtimeframe_set)
 	{
 		struct tm acttm;
 		localtime_r(&now, &acttm);
-		int32_t curtime = (acttm.tm_hour * 60) + acttm.tm_min;
-		int32_t mintime = client->allowedtimeframe[0];
-		int32_t maxtime = client->allowedtimeframe[1];
-		if(!((mintime <= maxtime && curtime > mintime && curtime < maxtime) || (mintime > maxtime && (curtime > mintime || curtime < maxtime))))
+		int32_t curday = acttm.tm_wday;
+		char *dest = strstr(weekdstr,"ALL");
+		int32_t all_idx = (dest - weekdstr)/3;
+		uint8_t allowed=0;
+		
+		// checkout if current time is allowed in the current day
+		allowed = CHECK_BIT(client->allowedtimeframe[curday][acttm.tm_hour][acttm.tm_min/30], (acttm.tm_min % 30));
+		
+		// or checkout if current time is allowed for all days
+		allowed |= CHECK_BIT(client->allowedtimeframe[all_idx][acttm.tm_hour][acttm.tm_min/30], (acttm.tm_min % 30));
+		
+		if(!(allowed))
 		{
 			er->rc = E_EXPDATE;
 		}
-		cs_log_dbg(D_TRACE, "Check Timeframe - result: %d, start: %d, current: %d, end: %d\n", er->rc, mintime, curtime, maxtime);
+		cs_log_dbg(D_TRACE, "Check Timeframe - result: %d, day:%s time: %02dH%02d, allowed: %s\n", er->rc, shortDay[curday], acttm.tm_hour, acttm.tm_min, allowed ? "true" : "false");
 	}
 
 	// user disabled
@@ -2449,15 +2494,15 @@ int32_t ecmfmt(char *result, size_t size, uint16_t caid, uint16_t onid, uint32_t
 	{
 		if(tier && payload)
 		{
-			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:0F06%.06s:%s", caid, prid, chid, srvid, l, ecmd5hex, payload, tier);				
+			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:0F06%s:%s", caid, prid, chid, srvid, l, ecmd5hex, payload, tier);
 		}
 		else if(tier)
 		{
-			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:%s", caid, prid, chid, srvid, l, ecmd5hex, tier);				
+			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:%s", caid, prid, chid, srvid, l, ecmd5hex, tier);
 		}
 		else if(payload)
 		{
-			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:0F06%.06s", caid, prid, chid, srvid, l, ecmd5hex, payload);	
+			return snprintf(result, size, "%04X@%06X/%04X/%04X/%02X:%s:0F06%s", caid, prid, chid, srvid, l, ecmd5hex, payload);
 		}
 		else
 		{ 
@@ -2683,6 +2728,7 @@ int32_t format_ecm(ECM_REQUEST *ecm, char *result, size_t size)
 		{ return ecmfmt(result, size, ecm->caid, ecm->onid, ecm->prid, ecm->chid, ecm->pid, ecm->srvid, ecm->ecmlen, ecmd5hex, csphash, cwhex, ecm->gbox_ecm_id, 0, payload, tier); }
 	else
 #endif
-		return ecmfmt(result, size, ecm->caid, ecm->onid, ecm->prid, ecm->chid, ecm->pid, ecm->srvid, ecm->ecmlen, ecmd5hex, csphash, cwhex, 0, 0, payload, tier);
+		return ecmfmt(result, size, ecm->caid, ecm->onid, ecm->prid, ecm->chid, ecm->pid, ecm->srvid, ecm->ecmlen, ecmd5hex, csphash, cwhex, 0,
+		((ecm->selected_reader && ecm->selected_reader->currenthops) ? ecm->selected_reader->currenthops : 0), payload, tier);
 }
 
