@@ -117,7 +117,7 @@ static void reader_log_emm(struct s_reader *reader, EMM_PACKET *ep, int32_t coun
 		if(!tps)
 			{ tps = &tpe; }
 
-		rdr_log(reader, "%s emmtype=%s, len=%d (hex: 0x%02X), cnt=%d: %s (%"PRId64" ms)",
+		rdr_log(reader, "%s emmtype=%s, len=%d (hex: 0x%.2X), cnt=%d: %s (%"PRId64" ms)",
 				username(ep->client), typedesc[ep->type], SCT_LEN(ep->emm)-3, SCT_LEN(ep->emm)-3, count, rtxt[rc], comp_timeb(&tpe, tps));
 	}
 
@@ -133,15 +133,19 @@ static void reader_log_emm(struct s_reader *reader, EMM_PACKET *ep, int32_t coun
 	{
 	case 0:
 		reader->emmerror[ep->type]++;
+		reader->webif_emmerror[ep->type]++;
 		break;
 	case 1:
 		reader->emmwritten[ep->type]++;
+		reader->webif_emmwritten[ep->type]++;
 		break;
 	case 2:
 		reader->emmskipped[ep->type]++;
+		reader->webif_emmskipped[ep->type]++;
 		break;
 	case 3:
 		reader->emmblocked[ep->type]++;
+		reader->webif_emmblocked[ep->type]++;
 		break;
 	}
 #endif
@@ -455,13 +459,13 @@ void do_emm(struct s_client *client, EMM_PACKET *ep)
 
 		if (aureader->fix_07 == 1 && ep->type == UNIQUE)
 		{
-			if((caid == 0x098C || caid == 0x09C4) && ep->emm[1] == 0x70 && (ep->emm[8] * 0x100 + ep->emm[9] != 0x200))
+			if((caid == 0x098D || caid == 0x098C || caid == 0x09C4) && ep->emm[1] == 0x70 && (ep->emm[8] * 0x100 + ep->emm[9] != 0x200))
 			{
 				rdr_log(aureader,"emmtype 0x%04X marked as unknown for caid 0x%04X", (ep->emm[8] * 0x100 + ep->emm[9]),caid);
 				ep->type = UNKNOWN;
 			}
 
-			if((caid == 0x098C || caid == 0x09C4) && ep->emm[1] == 0 && (ep->emm[4] * 0x100 + ep->emm[5] != 0x200))
+			if((caid == 0x098D || caid == 0x098C || caid == 0x09C4) && ep->emm[1] == 0 && (ep->emm[4] * 0x100 + ep->emm[5] != 0x200))
 			{
 				rdr_log(aureader,"emmtype 0x%04X marked as unknown for caid 0x%04X", (ep->emm[4] * 0x100 + ep->emm[5]),caid);
 				ep->type = UNKNOWN;
@@ -503,6 +507,7 @@ void do_emm(struct s_client *client, EMM_PACKET *ep)
 		{
 #ifdef WEBIF
 			aureader->emmblocked[ep->type]++;
+			aureader->webif_emmblocked[ep->type]++;
 			is_blocked = aureader->emmblocked[ep->type];
 #endif
 			/* we have to write the log for blocked EMM here because
@@ -532,7 +537,7 @@ void do_emm(struct s_client *client, EMM_PACKET *ep)
 
 		int32_t writeemm = 1; // 0= dont write emm, 1=write emm, default = write
 		
-		if(aureader->cachemm && !caid_is_irdeto(caid)) //Check emmcache early:
+		if(aureader->cachemm && !(caid_is_irdeto(caid) || caid_is_videoguard(caid))) //Check emmcache early:
 		{
 			unsigned char md5tmp[MD5_DIGEST_LENGTH];
 
@@ -592,7 +597,7 @@ int32_t reader_do_emm(struct s_reader *reader, EMM_PACKET *ep)
 
 	cs_ftime(&tps);
 	uint16_t caid = b2i(2, ep->caid);
-	if(reader->cachemm && !caid_is_irdeto(caid))
+	if(reader->cachemm && !(caid_is_irdeto(caid) || caid_is_videoguard(caid)))
 	{
 		MD5(ep->emm, SCT_LEN(ep->emm), md5tmp);
 		int64_t gone = comp_timeb(&tps, &last_emm_clean);
